from typing import List
from rocolib.utils.roco2sim.Node import *


scale = 0.001

class WrlFormatter:
    def __init__(self):
        self.COORD_FACET = """{face[0]:g} {face[1]:g} {face[2]:g}, """
        self.COORD_FACET_INDEX = """{face_pt_num[0]}, {face_pt_num[1]}, {face_pt_num[2]}, -1, """
        self.FACET_NORMAL = """0 0 0, """

    def coordPointsFormatter(self, p):
        self.coordPoints = []
        for x in p:
            self.coordPoints.append(self.COORD_FACET.format(face=x.astype(float)+0))
        return ''.join(self.coordPoints).strip(', ')

    def normalFormatter(self,n):
        self.normal = []
        for x in n:
            self.normal.append(self.FACET_NORMAL.format(face=x))
        return ''.join(self.normal).strip(', ')

    def coordIndexFormatter(self,c):
        self.coordIndex = []
        for x in c:
            self.coordIndex.append(self.COORD_FACET_INDEX.format(face_pt_num = [int(y) for y in x]))
        return ''.join(self.coordIndex).strip(', ')


def wboShape(sNode: ShapeNode):
    appearance = wboAppearance(sNode.appearance)
    name = wboName(sNode.name)
    geometry = wboGeometryIndexedFaceSet(sNode)
    node = """DEF %(name)s Shape {%(appearance)s %(geometry)s}""" % locals()
    return node


def wboGeometryIndexedFaceSet(sNode: ShapeNode):
    wrl = sNode.wrl
    if not wrl.coord:
        return ""

    wf = WrlFormatter()
    normal = wf.coordPointsFormatter(wrl.normal)
    points = wf.coordPointsFormatter(wrl.coord)
    coordIndex = wf.coordIndexFormatter(wrl.index)

    node = """geometry IndexedFaceSet {coord Coordinate {point [%(points)s]} normal Normal {vector [%(normal)s]} coordIndex [%(coordIndex)s]}""" % locals()
    return node


def wboAppearance(aNode: AppearanceNode):
    baseColor = str(aNode.getRGB()).strip('[ ]')
    roughness = str(aNode.getRoughness())
    metalness = str(aNode.getReflection())
    name = wboName(aNode.getName())
    node = """appearance PBRAppearance { baseColor %(baseColor)s roughness %(roughness)s metalness %(metalness)s name "%(name)s"}""" % locals()
    return node


def wboPhysics(pNode: PhysicsNode):
    if pNode is None:
        return """ """
    density = pNode.material.density
    mass = pNode.mass
    node = """physics Physics {density %(density)s mass %(mass)s}""" % locals()
    return node


def wboXyz(sNode: ShapeNode):
    return str(scale * np.array(sNode.getXyz()).astype(int)[0:3, 3]).strip('[ ]')


def wboSolid(sNode: ShapeNode):
    if not sNode.isSolid():
        return wboShape(sNode)
    shape = wboShape(sNode)
    contactMaterial = wboContact(sNode)
    physics = wboPhysics(sNode.getPhysicsNode())
    name = wboName(sNode.name)
    node = """DEF %(name)s Solid {children [%(shape)s] %(contactMaterial)s boundingObject USE %(name)s %(physics)s}""" % locals()
    return node

def wboContact(sNode: ShapeNode):
    contactMaterial = sNode.getPhysicsNode().contact
    if contactMaterial is None:
        return """contactMaterial "default" """
    return """contactMaterial "%(contactMaterial)s" """


def wboRobot(mNode: ShapeNode):
    # create local var to fill the nodes
    name = wboName(mNode.name)
    mNode.gasify()
    solid = wboSolid(mNode)
    mNode.children.append(solid)
    childrenNode = " ".join(mNode.children)
    physics = wboPhysics(mNode.getPhysicsNode())
    contactMaterial = wboContact(mNode)
    rotation = str(mNode.getRotationVector()).strip('[ ]')
    # battery = mNode.getDeviceOptions('battery')
    # cpuConsumption = mNode.getDeviceOptions('cpuConsumption')
    node = """DEF %(name)s Robot {
  rotation %(rotation)s
  children [%(childrenNode)s]
  name "%(name)s"
  %(contactMaterial)s
  boundingObject USE %(name)s
  %(physics)s
  controller "%(name)s"
  supervisor TRUE
}""" % locals()
    return node


def wboHeader(version="R2021b"):
    return """#VRML_OBJ R2021b utf8
"""


def wboConverter(sNodes: List[ShapeNode]):
    robot = sNodes[0]
    for sNode in sNodes[1:len(sNodes)]:
        robot.children.append(wboSolid(sNode))
    node = wboHeader() + wboRobot(robot)
    return node


def wboName(name):
    return name.replace('.', '_')
